//-------------------------------------------------
// TNCPIC.C
//
// Gateway AX25 pour Alan (PIC16F873 @ 4Mhz)
//
// Created by : Remi Bilodeau VE2YAG 10/09/2000
// Last Modification : 10/09/2000  Version 1.0 built 01
//
// ALAN Protocol was created by cooperation with Pierre Brouillard VE2PBO
// This is For Use ONLY in HAM Radio Applications.
//
//
// History:
// 10/09/2000  V1.0b01  Initial version, not tested on-the-air
// 11/23/2000  V1.1b01  Use shematic rev B.
//                      -RXD now wired to RB0
//                      -CPU activity led on RB5
//                      -PTT on RB4
//                      -Flush all i/o to keep only modem
//                      -Disable RCIE when alan transmit
//
// 11/11/2001  V1.2     Some hardware change
//                      -PTT combined with MODE0 on RB2
//                      -CPU led on RB4
//                      Added option
//                      -Alan NET/ID in eeprom - !Ixx !Nxx command
//
// Pinout:
// TMR0 Alan and AX25 timing
//
// RA0
// RA1
// RA2
// RA3
// RA4
// RA5
// RB0 TONE IN (from modem)
// RB1 TONE OUT (to modem)
// RB2 MODE0/PTT (for MX614 chip)
// RB3 MODE1 (for MX614 chip)
// RB4 CPU activity led
// RB5
// RB6
// RB7
// RC0
// RC1
// RC2
// RC3
// RC4
// RC5 RS485 gate control
// RC6 RS485 TX (to network)
// RC7 RS485 RX (from network)
//
//
// Reste a faire:
// Net et ID dans EEPROM
// utiliser watchdog
//
//-------------------------------------------------
#include <pic.h>

/*{{{  Define*/

// Minimum speed for packet in 4Mhz, Max speed is 19Mhz not 20Mhz.
// Timer overflow occur if you use 20Mhz, until you change timer
// divider.

#define Xtal   4000000	// Processor Crystal Speed in Hz
#define FALSE 	0       // Value of False
#define TRUE	!FALSE  // Value of True

#define	PORTBIT(adr, bit)	((unsigned)(&adr)*8+(bit))
#define CLEARWATCHDOG() asm("clrwdt")

static bit	LED      @	PORTBIT(PORTB, 4);

void PacketStart();
void PacketSendByte();
void PacketEnd();
void Transmit(unsigned char);
void AlanHeader(unsigned char);
void AlanFooter(void);
void AlanCmd(unsigned char);
void AlanDevId(void);
void AlanReceive(void);
void AlanPoll(void);
void Setup_Alan_Mode(void);/*}}}  */

/*{{{  variable*/

//-----------------------------------------
// Packet buffer size, buffer and pointer
//-----------------------------------------
#define PKTBUFSIZE 58
unsigned char PktBuf[PKTBUFSIZE];   // Packet buffer
unsigned char p,tmp;                // p is pointer in PktBuf buffer tmp is general temporary (ADC read and main)
/*}}}  */

/*{{{  EEPROM memory map*/

// EEPROM MEMORY MAP
//--------------------
#define EE_ALANID     0         // Alan ID
#define EE_ALANNET    1         // alan NET


/*}}}  */


/*{{{  Internal function*/

void EEPromWrite(addr, value) {
    while(WR)continue;
    EEADR=(addr);
    EEDATA=(value);
    GIE=0;
    WREN=1;
    EECON2=0x55;
    EECON2=0xAA;
    WR=1;
    WREN=0;  
    GIE=1;
}

unsigned char EEPromRead(unsigned char addr) {
    return ((EEADR=(addr)),(RD=1),EEDATA);
}


unsigned char Hex(unsigned char c) {
    if(c>='0' && c<='9') return c-'0';
    if(c>='A' && c<='F') return c-('A'-10);
    if(c>='a' && c<='f') return c-('a'-10);
    return 0;
}

void ADCRead(unsigned char channel) {
	ADCON0 = (channel << 3) + 0x81;		   // Enable ADC, OSC., and selected chan.
	for(tmp=0; tmp<20; tmp++) asm("nop");  // Minimum hold time 2Tad (19us)
	ADGO = 1;                              // Start Conversion
	while(ADGO)	continue;				   // Wait for conversion complete...
}
/*}}}  */


/*{{{  Packet module*/

//________________________________________________________________________
//
// Packet Receiving Code   by Byon Garrabrant  N6BG  byon@mail.com  Apr 99
//________________________________________________________________________
//
//   This software is  available only for non-commercial  amateur radio
//   or educational  applications.   ALL  other  uses  are  prohibited.
//   This  software may  be  modified  only  if the  resulting code  be
//   made available  publicly  and the  original  author given  credit.
//   Contact the author or TAPR for information on commercial licenses.
//   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved.
//________________________________________________________________________
//
// Remi Bilodeau 18 sept. 2000
// Plusieurs erreurs dans le code de reception et de transmission packet:
// -Le bit stuffering en TX ne fonctionne pas correctement
// -Le timing de reception est deficient, j'ai modifier pour pouvoir
//  calculer le temps exact entre 2 transitions pour determiner le
//  bit length au lieu de verifier l'etat de la pin d'input 1200 fois
//  par seconde ce qui peut se desyncroniser tres facilement. une
//  transition de longueur 1 va de 0.5 a 1.5 bit lenght, un bit 2 de 1.5
//  a 2.5 etc... Ca s'autosyncronise et accepte un baud rate pas tout a
//  fait de 1200 baud.
//
//  L'optimisation du compiler cause un paquet de trouble, il supprime des
//  instructions assembleur a verifier en cas de compilation avec autre que
//  V7.86pl3.  Les variable doivent etre en bank 0.
//
//  Init:     PacketInit
//  Transmit: PacketStart, PacketSendByte(...), PacketEnd
//  Receive:  PacketIn_Update (Check flag NewByte, Done and Reset)
//                             Get data in PacketIn_Byte and reset flag
//
//  Routine tester a 4Mhz sans probleme.
//  Avec timer diviser par 16, max speed is 19Mhz
//________________________________________________________________________


/*{{{  Packet variable*/


#define PKTBAUD Xtal/4/16/1200

static bit  TONE_IN  @	PORTBIT(PORTB, 0);  // Change in assembly if you modify
static bit	TONE     @	PORTBIT(PORTB, 1);  // Change in assembly if you modify
static bit	MODE0    @	PORTBIT(PORTB, 2);
static bit	MODE1    @	PORTBIT(PORTB, 3);
//static bit	PTT      @	PORTBIT(PORTB, 4);


//------------------------------------------------------------------------------
// Packet transmit variable
//------------------------------------------------------------------------------
#define SYNCFLAG 40

unsigned char packet_sending_char,packet_sending_count,packet_bit_history;
unsigned char packet_crc_hi,packet_crc_lo,txdelay,IsFlag;


const unsigned char Pkt_Header[] = {'A'<<1,'L'<<1,'A'<<1,'N'<<1,' '<<1,' '<<1,0x60, \
	                                'V'<<1,'E'<<1,'2'<<1,'V'<<1,'A'<<1,'G'<<1,0x61, \
                                	0x03,0xF0,0 };


//------------------------------------------------------------------------------
// Packet receive(Sharing with transmit variable)
//------------------------------------------------------------------------------
// Define also affect assembly function
#define PacketIn_Flag       packet_sending_char
#define PacketIn_Count      packet_sending_count
#define PacketIn_Flag_Count packet_bit_history
#define PacketIn_Data       packet_crc_lo
#define PacketIn_Byte       packet_crc_hi
#define PacketIn_crc_lo     txdelay
#define PacketIn_crc_hi     IsFlag
#define _PacketIn_Flag       _packet_sending_char
#define _PacketIn_Count      _packet_sending_count
#define _PacketIn_Flag_Count _packet_bit_history
#define _PacketIn_Data       _packet_crc_lo
#define _PacketIn_Byte       _packet_crc_hi
#define _PacketIn_crc_lo     _txdelay
#define _PacketIn_crc_hi     _IsFlag
unsigned char PacketIn_Size;

// Flag definition to PacketIn_Flag
#define PKTIN_NEWBYTE 1
#define PKTIN_RESET 2
#define PKTIN_DONE 4
/*}}}  */

/*{{{  PacketInit*/

void PacketInit() {
    MODE1 = 1;              // Setup modem
    MODE0 = 0;
//    PTT = 0;
    PacketIn_Flag = 0;      // Receive flag
}
/*}}}  */

/*{{{  PacketSendByte*/

//-----------------------------------------------
// Send Byte, Byte in packet_sending_char
//-----------------------------------------------
void PacketSendByte() {
#asm
#define T0IF  2
#define clrc   bcf 3,0
#define setc   bsf 3,0
#define skipc  btfss 3,0
#define skpnc  btfsc 3,0
#define skipz  btfss 3,2
#define skipnz btfsc 3,2

    bcf     _STATUS, 5                  ; Select BANK 0
    bcf     _STATUS, 6
	movlw	8			                ; send 8 bits
	movwf	_packet_sending_count

Packet_Send_Loop
	rrf	    _packet_sending_char, W     ; save bit for CRC
	movlw	1
	skpnc
	xorwf	_packet_crc_lo       	    ; crc ^= d & 1;
	clrc
	rrf	    _packet_crc_hi   	        ; crc >>= 1
	rrf	    _packet_crc_lo
	skipc
	goto	Packet_CRC_Done	            ; if shifted off 1
	movlw	0x08			            ; xor POLY 0x8408
	xorwf	_packet_crc_lo
	movlw	0x84
	xorwf	_packet_crc_hi
Packet_CRC_Done
	rrf	    _packet_sending_char 	    ; get bit again for sending
    /*{{{  SendBit*/

PacketSendBit
	rlf	    _packet_bit_history  	; store the last 8 bits sent
	movlw	PKTBAUD	    		    ; bump TMR0 forward forward
	subwf	_TMR0   		        ; 1200 baud updates
	rrf	    _packet_bit_history, W	; restore the bit in carry
	btfss	_INTCON, T0IF		    ; wait for last bit to be sent
	goto	$-1
	bcf	    _INTCON, T0IF		    ; clear TMR0 overflow
	skpnc               	        ; if sending a 1
    goto    bstuff                  ; check for bit stuffering

	movlw	0x02             		; toggle TONE_OUT
	xorwf	_PORTB
    goto    endbit

bstuff
    movf    _IsFlag, W              ; Verify if transmit flag
    skipz
    goto    endbit
	comf	_packet_bit_history, W	; check the last 5 bits
	andlw	0x1F	        		; for bit stuffing
 	skipz                           ; if not all 1s
    goto    endbit		            ; return
	clrc                		    ; send a transparent 0 bit
	goto	PacketSendBit

endbit
/*}}}  */

    decfsz	_packet_sending_count   	; continue for all 8 bits
	goto	Packet_Send_Loop
#endasm
}
/*}}}  */

/*{{{  PacketStart*/

void PacketStart() {
    RCIE = 0;
    MODE1=0; MODE0=1;       //PTT=1;
    txdelay=SYNCFLAG;
    IsFlag=1;               // Send sync flag

    while(--txdelay) {
        packet_sending_char=0x7E;
        PacketSendByte();
    }
    IsFlag=0;

    packet_crc_lo=0xFF;     // Reset CRC
    packet_crc_hi=0xFF;

    while(1) {              // Send packet header
        packet_sending_char = Pkt_Header[txdelay++];
        if(!packet_sending_char) break;
	    PacketSendByte();
    }
}
/*}}}  */

/*{{{  PacketEnd*/

void PacketEnd() {
    packet_sending_char = ~packet_crc_lo;
    txdelay             = ~packet_crc_hi;

    PacketSendByte();                   // Send CRC lo
    packet_sending_char = txdelay;
    PacketSendByte();                   // Send CRC hi
    IsFlag=1;
    packet_sending_char=0x7E;
    PacketSendByte();                   // Send two final flag
    packet_sending_char=0x7E;
    PacketSendByte();
    PacketInit();                       // Reset packet module
    RCIE=1;
}
/*}}}  */


/*{{{  PacketIn_Update*/

void PacketIn_Update() {
#asm
#define	PacketInNewByteFlag	_PacketIn_Flag, 0
#define	PacketInResetFlag	_PacketIn_Flag, 1
#define	PacketInDoneFlag	_PacketIn_Flag, 2
#define	PacketInActiveFlag	_PacketIn_Flag, 3
#define	PacketInValidFlag 	_PacketIn_Flag, 4
#define PacketLastBitState  _PacketIn_Flag, 5

    bcf     _STATUS, 5          ; Select BANK 0
    bcf     _STATUS, 6
    btfsc   PacketLastBitState  ; Test if bit has changed and
    goto    LastIsOne           ; Update last bit state
    btfss   _PORTB,0
    goto    UpdateCount
    bsf     PacketLastBitState
    goto    BitHasChange

LastIsOne
    btfsc   _PORTB,0
    goto    UpdateCount
    bcf     PacketLastBitState
    goto    BitHasChange

UpdateCount                     ; Bit not change, update bit counter and timer.
	btfss	_INTCON, T0IF		; if not timer wrap
    return 			            ;  done
	bcf	    _INTCON, T0IF	    ; clear timer wrap
	movlw	PKTBAUD		        ; bump timer forward
	subwf	_TMR0, F
	incf	_PacketIn_Count, F  ; Increment count
    return

BitHasChange                    ; Bit has change, process it.
    movlw   256-(PKTBAUD/2)     ; Timer set to 1/2 bit delay
    movwf   _TMR0

    movf	_PacketIn_Count, W
	clrf	_PacketIn_Count		; packetin_count = 0

    sublw   0                   ; 0-Count
	addlw   1                   ; Bit 1-7 is valid, 0 is too fast
    skipnz
	goto	PacketIn_Add0
	addlw   1
	skipnz
	goto	PacketIn_AddOne
	addlw   1
	skipnz
	goto	PacketIn_AddTwo
	addlw   1
	skipnz
	goto	PacketIn_AddThree
	addlw   1
	skipnz
	goto	PacketIn_AddFour
	addlw   1
	skipnz
	goto	PacketIn_6
	addlw   1
	skipnz
	goto	PacketIn_Flag

PacketIn_Invalid
	btfsc	PacketInActiveFlag	; if we were active,
	bsf	    PacketInResetFlag	;  reset
	bcf	    PacketInActiveFlag	; we are no longer active
	bcf	    PacketInValidFlag
	clrf	_PacketIn_Flag_Count
	clrf	_PacketIn_Count		; packetin_count = 0
	return

PacketIn_6
	call	PacketIn_Add1       ; Add five '1' bits and remove '0' bit.
	call	PacketIn_Add1
	call	PacketIn_Add1
	call	PacketIn_Add1
	goto	PacketIn_Add1

PacketIn_AddFour                ; Add '1' bit(s) and after add the '0' bit
	call	PacketIn_Add1
PacketIn_AddThree
	call	PacketIn_Add1
PacketIn_AddTwo
	call	PacketIn_Add1
PacketIn_AddOne
	call	PacketIn_Add1

PacketIn_Add0
	clrc
	goto	PacketIn_Add
PacketIn_Add1
	setc
	clrf	_PacketIn_Flag_Count	; we are not in flags
PacketIn_Add
	rrf	    _PacketIn_Data, F	; save the bit
	rlf	    _PacketIn_Data, W	; restore the bit
	movlw	1
	skpnc
	xorwf	_PacketIn_crc_lo, F	; crc ^= d & 1;
	clrc
	rrf	    _PacketIn_crc_hi, F	; crc >>= 1
	rrf	    _PacketIn_crc_lo, F
	skipc
	goto	PacketIn_CRC_Done	; if shifted off 1
	movlw	0x08    			;  xor POLY 0x8408
	xorwf	_PacketIn_crc_lo, F
	movlw	0x84
	xorwf	_PacketIn_crc_hi, F
PacketIn_CRC_Done

	btfss	PacketInValidFlag
	return
	nop
	decfsz	_PacketIn_Size, F
	return
	movf	_PacketIn_Data, W
	bsf	    PacketInActiveFlag	; we are active
	bsf	    PacketInNewByteFlag	; got a new incoming byte
	movwf	_PacketIn_Byte
	movlw	8			        ; reset size count to 8
	movwf	_PacketIn_Size
	return

PacketIn_Flag
	movlw	8
	movwf	_PacketIn_Size
	btfss	PacketInValidFlag	; if valid
	goto	PacketIn_Not_Valid

	btfss	PacketInActiveFlag	; if active,
	goto	PacketIn_Not_Active

	movlw	0x78			        ; compare CRC to H'785C'
	xorwf	_PacketIn_crc_hi, W
	skipz				            ; if bad
	bsf	    PacketInResetFlag	    ;  reset
	movlw	0x5C
	xorwf	_PacketIn_crc_lo, W
	skipz				            ; if bad
	bsf	    PacketInResetFlag	    ;  reset

	btfss	PacketInResetFlag	    ; if not reset
	bsf	    PacketInDoneFlag	    ;  packet is valid, we are done
	bcf	    PacketInActiveFlag	    ; we are no longer active
	bcf	    PacketInValidFlag	    ; we are not valid
	clrf	_PacketIn_Flag_Count	; we are not in flags
	return

PacketIn_Not_Valid			        ; if not valid
	incf	_PacketIn_Flag_Count, F	; count flags til valid
	movf	_PacketIn_Flag_Count, W
	xorlw	3			            ; if 3rd flag in a row
	skipnz
	bsf	    PacketInValidFlag	    ; we are valid

PacketIn_Not_Active			        ; if not active
	movlw	0xFF			        ; reset crc to 0xFFFF
	movwf	_PacketIn_crc_hi
	movwf	_PacketIn_crc_lo
	return
#endasm
}
/*}}}  */

/*}}}  */

/*{{{  Alan module*/
/* ---------------------------------------------------------------
 * Alan Core for PIC16C7x Processors (SCI) for HT-PIC Compiler
 * Created by : P. Brouillard 07/98 VE2 PBO
 * Modified by: Remi Bilodeau 11/2001 VE2YAG
 *
 * Last Modification : 14/01  Version 1.4 built 01
 *
 * ALAN Protocol was created by cooperation with Remi Bilodeau VE2YAG
 * This is For Use ONLY in HAM Radio Applications.
 *
 * ---History---
 * Date		Version	Description
 *  1 Jul 98	1.0	 Initial Creation, HT-PIC Version
 *  5 Jul 98  1.1  Major Tune-up and Cleanup
 *  1 Dec 99  1.2  Support for Alan Protocol Version 1.1
 *                 AlanReceive() is now running in Interrupt Service
 * 14 Jan 00  1.2a Added Network ID Support
 * 14 Jan 00  1.2b Added lockout If a DTA Packet is received and application
 *                 is not ready, then it will send NEG Packet
 * 14 Jan 00  1.2c Added 2 Register for Size of Data and Start pointer
 *                 for accessing data in a DTA packet type.
 * 8  oct 00  1.3  Alan transmission also go through receive buffer for
 *                 dual port alan protocol. By Remi Bilodeau
 * 14 oct 01  1.4  Variable cleanup, due to dual buffer usage (Rx and Tx)
 *
 */

#define	NOP()	asm(" nop") // Definition of a NOP Instruction
#define Z 2					// Define Position of ZERO  flag in STATUS Register
#define C 0     			// Define Position of CARRY flag in STATUS Register

// Structure to Access a Bit in a Byte (8 Bit)
struct bbit {
       unsigned b0 :1;	// bit 0
       unsigned b1 :1;  // bit 1
       unsigned b2 :1;  // bit 2
       unsigned b3 :1;  // bit 3
       unsigned b4 :1;  // bit 4
       unsigned b5 :1;  // bit 5
       unsigned b6 :1;  // bit 6
       unsigned b7 :1;  // bit 7
       };

// Union to call the byte holding the bits
union bitaccess { struct bbit bt ; unsigned char all; };


/*{{{  Alan Definitions*/
// --- Alan Hard & Soft Definitions

/* NOTE: Size your 'ALANBUFFER_SIZE' with a Minimum of 8 bytes */
#define ALANBUFFER_SIZE 39      // Reserve x Byte for Alan Rx Buffer
#define RS485Gate	    RC5	    // RS485 Direction output  1 = TX
#define ALAN_IDENT_SIZE 8	    // Set lengh of ID Message
static const unsigned char AlanIDMsg[] = "PTNC0112";   // 8 char ID Message

/*--- Alan Serial I/O Definition ---*/
#define ARxFull  RCIF       // PIR1.RCIF - Rx Serial Buffer Full
#define ATxEmpty TXIF       // PIR1.TXIF - Tx Serial Buffer Empty
#define ARx_Chr  RCREG      // Serial Receive Register
#define ATx_Chr  TXREG      // Serial Transmit Register
#define ANoTX	 TRMT       // TXSTA.TRMT - No Byte in Transmit Register
#define ARxLK    OERR       // RCSTA.OERR - Receiver Overrun (Lockup)
#define ARxEn    CREN       // RCSTA.CREN - Receiver Enable

/*--- Define Standard ALAN Command Op-Codes ---*/
#define Alan_ACK  	0x00	// Acknowledges that data was received
#define Alan_REP  	0x08	// Ask for a Device Status Report
#define Alan_CANA 	0x10	// Cancel Alert Generated by device
#define Alan_A_ID 	0x18	// Ask for Device Identification
#define Alan_RST  	0x20	// Used to Reset Device (User Defined)
#define Alan_PING 	0x28 	// Use to Detect Presence of Device
#define Alan_PONG 	0x30	// Send if Device Receive a PING
#define Alan_NEG	0x38	// Negate, Alan Data or Command was refused
#define Alan_DTA	0x80	// Data Packet
#define Alan_STA	0x88	// Status Packet
#define Alan_ID		0x90	// Device Identification
#define Alan_ALRT	0x98	// Alert Packet

/*--- Baud Rate Selection for Alan Network ---*/
#define ABaud (Xtal/(2400L*64L))-1L
/*}}}  */

/*{{{  Alan Variables*/
// --- Alan Variables & Prototypes

static bank1 unsigned char Alantmp;		// Alan Temporary variable
static bank1 unsigned char AlanMyID;    // Active Alan Device ID
static bank1 unsigned char AlanMyNET; 	// Active Alan Device NET

static bank1 unsigned char Acksum;		// Alan Packet Checksum for Tx/Rx
static bank1 unsigned char AlanBufSize; // # of Received byte to process
static bank1 unsigned char AlanBuffer[ALANBUFFER_SIZE];	// Reserve Memory for Receive Alan Packet
static bank1 unsigned char AdtaSize;    // (for App) Size of the DATA in Alan DTA Packet

static bank1 unsigned char ATxDestID;   // Transmit Packet Destination Device Number...
static bank1 unsigned char ATxDestNET;  // Transmit Packet To Destination NET Number...

struct AFlag {
       unsigned AlanRx   :1;	// Bit Set when Header is Detected      (Internal)
       unsigned AEscape  :1;	// Bit Set when Escape Char is Detected (Internal)
       unsigned ANewDta  :1;    // Bit Set when New DTA is Received
       unsigned AValid   :1;	// Bit set when DTA Packet is Valid
       unsigned ARxAck   :1;	// Bit Set when Acknowledge was Received
       unsigned ARep 	 :1;	// Bit Set when Report Packet was Received
       unsigned AReset   :1;	// Bit Set when Reset Packet was Received
       unsigned AAlrt    :1;	// Bit Set when Alert Packet is Activated
       };

union AlanStaFlags { struct AFlag AlanFlag; volatile unsigned char AlanFlgReg; };
union AlanStaFlags AlanSta;

/* --- Definition of Reserved Memory Locations --- */

#define ARxDestID  AlanBuffer[0] // Packet To Destination Device Number...
#define ARxDestNET AlanBuffer[1] // Packet To Destination NET Number...
#define ARxSrcID   AlanBuffer[2] // Packet From Device Number
#define ARxSrcNET  AlanBuffer[3] // Packet From NET Number
#define ARxCmd     AlanBuffer[4] // Pck Command
#define ARxSize    AlanBuffer[5] // Pck Data Size (DTA Packet type)
/*}}}  */


/*{{{  Interrupt Routine*/
/* ---------------------------------------------------------------
 * Interrupt Routine
 * Definitions Created by : P. Brouillard 07/98 VE2 PBO
 * Last Modification : 08/2000  Version 1.2a built 01
 */

void interrupt tc_int(void) {
  if(RCIF) {                        // RX Serial Full Interrupt
    RCIF = 0;                       // Clear interrupt flag
    AlanReceive();
  }
}
/*}}}  */

/*{{{  Transmit*/
/* --- Send Data Bytes with ALAN Protocol Specification ---
 * Byte to Transmit in W on routine entry, Send byte to Serial transmit
 * buffer with a Escape Sequence if a value of 255 or 254 is detected.
 * Checksum is calculated at each time routine is called.
 */

void Transmit(unsigned char aSerOutDat) {
    if(AlanBufSize==(ALANBUFFER_SIZE-1)) return;
    AlanBuffer[AlanBufSize++] = aSerOutDat;
    Acksum+=aSerOutDat;
}
/*}}}  */

/*{{{  AlanHeader*/
/* --- Send ALAN Header ---
 * This Routine will send ALAN Header compose of Start Char. (255),
 * Destination ID (DestID) and Network (DestNET) to sent packet to...
 * And with ID (SrcID) and Network (SrcNet) of this Node (this Processor)
 */

void AlanHeader(unsigned char AiCmd) {
    RCIE = 0;                     // Desactive receive interrupt
    AlanSta.AlanFlag.AlanRx = 0;  // Clear Alan Flags (cancel current reception)
    AlanBuffer[0]=ATxDestID;       // Setup header in buffer
    AlanBuffer[1]=ATxDestNET;
    AlanBuffer[2]=AlanMyID;
    AlanBuffer[3]=AlanMyNET;
    AlanBuffer[4]=AiCmd;
    AlanBufSize= 5;               // Set current buffer point

    Acksum = ATxDestID+ATxDestNET+AlanMyID+AlanMyNET+AiCmd;
}
/*}}}  */

/*{{{  AlanFooter*/
/* --- Send Calculated Checksum ---
 * This Routine transmit the checksum that complete the packet
 */

void AlanFooter(void) {

    AlanBuffer[AlanBufSize++] = 255-Acksum;

    //--------------------------------------
    // PACKET FOR REMOTE NETWORK
    //--------------------------------------
    if(ARxDestNET!=AlanMyNET) {
        PacketStart();
        packet_sending_char = 255; PacketSendByte();
        for(Alantmp=0; Alantmp<AlanBufSize; Alantmp++) {
            packet_sending_char = AlanBuffer[Alantmp]; PacketSendByte();
        }
        PacketEnd();
    	RCIF = 0;                   // Active receive interrupt
        return;
    }

    //--------------------------------------
    // PACKET FOR LOCAL NETWORK
    //--------------------------------------
	RS485Gate  = 1;	    		// Go in Transmit Mode ( Put RS-485 in TX )
	ATx_Chr = 255;				// Send Packet Header  (Send Direct Start Character)

    for(Alantmp=0; Alantmp<AlanBufSize; Alantmp++) {
        Acksum = AlanBuffer[Alantmp];
	    if(Acksum>253)	{		// Check for Escape Sequence (254 or 255)
	        Acksum-=254;
	        while(!ANoTX);	    // Loop until Serial Tx Buffer is empty
	        ATx_Chr = 254;		// Send data Direct (Escape Caracter)
        }
	    while(!ANoTX);		    // Loop until Serial Tx buffer is empty
	    ATx_Chr = Acksum;		// Send Data
    }
	while(!ANoTX);				// Loop Till TX Register is Full
	RS485Gate = 0;			    // Go in Receive Mode (Put RS-485 in RX)
	RCIF = 0;                   // Active receive interrupt
    RCIE = 1;

}
/*}}}  */

/*{{{  AlanCmd*/
/* --- Send Command ---
 * This Routine will Send a Command (in W on entry) to a Remote Device
 * Transmit Adress must be Set Before Calling Routine
 */

void AlanCmd(unsigned char AiiCmd) {
	AlanHeader(AiiCmd);	// Send Packet Header
	AlanFooter();				// Send Packet Footer
}
/*}}}  */

/*{{{  AlanDevId*/
/* --- Send Id String ---
 * This Routine will return Identification String to Device
 * that Requested Identification.
 */

void AlanDevId(void) {
    unsigned char i;

	AlanHeader(Alan_ID);	    // Send ID Command ( Identification Packet )
	Transmit(ALAN_IDENT_SIZE);	// Transmit Size of String

    // Transmit String Caracters
    for(i=0;i<ALAN_IDENT_SIZE;i++) Transmit(AlanIDMsg[i]);

	AlanFooter();					// Send Packet Footer
}
/*}}}  */

/*{{{  AlanReceive*/

/* --- Process Received Byte with ALAN Protocol Specification ---
 * This Routine will detect a Alan Start Header, Process a Escape
 * Sequence, Store Byte in the RX Buffer and Calculate Checksum
 */

void AlanReceive(void) {
	if(ARxLK) { 							// Reset SCI Receiver after Error
    	ARxEn  = 0;							// Disable Receiver
	    AlanSta.AlanFlag.AlanRx = 0;		// Reset Alan Decoder
    	ARxEn  = 1;							// Enable Receiver
	}


	while(1) {   							// Do until Serial Buffer FIFO is Empty
		if(!ARxFull) return;				// Return, If no more byte to Process
		Alantmp = ARx_Chr;					// Get Byte from Serial Port

		if(Alantmp==255) {	                // Detection of Packet Header
    	    AlanBufSize = 0;					    // Reset Rx Counter
		    Acksum = 0;						// Clear Rx Checksum
		    AlanSta.AlanFlag.AlanRx = 1;	// Receive Packet is now in Progress
	 	    AlanSta.AlanFlag.AEscape = 0;	// Reset - No Escape Sequence
		    return;													    // Return, Ready for Packet
		}

		if(!AlanSta.AlanFlag.AlanRx) return; 	// Return, No Packet Header Detected

		if(Alantmp==254) {			            // Detection of Escape Sequence
	  	    AlanSta.AlanFlag.AEscape = 1;		// Escape Sequence Detected
	  	    return;								// Return, Wait for Other Byte
		}

		if(AlanSta.AlanFlag.AEscape) {  		// Process Escape [254] + (x) = byte
    	    AlanSta.AlanFlag.AEscape = 0;		// Cancel Escape Sequence
	  	    if(Alantmp)							// Send if byte is 255
	    	    Alantmp=255;		            // Escape Sequence + 1 then byte is 255
    	    else
	    	    Alantmp=254;			        // Escape Sequence + 0 then byte is 254
		}

		if(AlanBufSize>=ALANBUFFER_SIZE) {             // Detection of Buffer Overflow
    	  	AlanSta.AlanFlag.AlanRx=0;			// Stop Receiving Packet to BIG for Buffer
	      	return;
		}

		AlanBuffer[AlanBufSize]=Alantmp;      // Store Data in Rx Buffer
		AlanBufSize++;						// Increment Buffer Counter
		Acksum += Alantmp;				// Calculate Receive Checksum
	}
}
/*}}}  */

/*{{{  AlanInit*/

/* --- Setup Serial Port of CPU for Alan Network ---
 * This Routine will Setup Serial Port for Alan Network
 */

void AlanInit(void) {
    RCIE = 0;
    RS485Gate  = 0;     // Clear RS-485 gate
    ATx_Chr = 0;		// Clear Transmitter Register
    ARx_Chr = 0;		// Clear Receiver Register
    SPBRG = ABaud;	    // Set Alan Baud Rate
    TXSTA = 0x20;		// Setup Serial Transmiter
    RCSTA = 0x90;		// Setup Serial Receiver

    AlanSta.AlanFlgReg = 0;	            // Clear Alan Flags

    AlanMyNET = EEPromRead(EE_ALANNET);	// Setup Starting Alan NET
    AlanMyID  = EEPromRead(EE_ALANID);  // Setup Starting Alan ID
    RCIE = 1;
}
/*}}}  */

/*{{{  AlanPoll*/
/* --- Service Routine to Run Alan Protocol on CPU ---
 * This Routine will process bytes if any, if present at RX Serial Port.
 * This routine will Check Received Packet Integrity (Checksum) and provide
 * Minimum Alan Protocol Calls (Ident [ID], Ping [PONG], Ack. [ACK])
 */

void AlanPoll(void) {

	if(!AlanSta.AlanFlag.AlanRx) return;	// Return, if no data to Process
	if(AlanBufSize<6) return;					// Return, Packet Header not Complete

	if(ARxCmd>0x7F)	{ 		                // Do if Packet has a Data Structure (CMD>127)
	    if(AlanSta.AlanFlag.AValid) {       // Check if Application is ready to receive a Data Packet
     	    AlanSta.AlanFlag.AlanRx = 0;    // Packet can't be Processed, Stop Receiving
            return;                         // Return, can't Process Data Packet
        }
	    Alantmp = ARxSize + 7;				// Calculate Size of Data Packet
	    if(Alantmp!=AlanBufSize) return;	// Return, if Data Structure is Incomplete
	}

    AlanSta.AlanFlgReg = 0;	                // Clear Alan Flags, Stop receiving


	// Extract Remote Checksum from Internal Checksum and Invert the Result
	Alantmp = 255-( Acksum - AlanBuffer[AlanBufSize-1] );
    if(AlanBuffer[AlanBufSize-1] != Alantmp) return; 	// Return, if Checksum is Bad


    //---------------------------------
    // ALAN PACKET IS FOR THIS DEVICE
    //---------------------------------
	if(ARxDestNET==AlanMyNET && ARxDestID==AlanMyID) {
        /*{{{  Packet for this device*/

  ATxDestID  = ARxSrcID;                  // Set Tx Dest ID  to adress of remote cpu
  ATxDestNET = ARxSrcNET;                 // Set Tx Dest NET to adress of remote cpu

switch(ARxCmd) {                          // Process Command in Received Packet

    case Alan_DTA:                        // Process a Data Packet
         AlanSta.AlanFlag.AValid = 1;     // Data Packet is Valid for End Application
         AlanSta.AlanFlag.ANewDta = 1;    // This is a New Data Packet for End Application
         AdtaSize = ARxSize;              // Size of Data in DTA Packet type
         break;                           // Packet Process Complete

    case Alan_PING:                       // Process a PING Packet
         AlanCmd(Alan_PONG);              // Send Back a PONG Packet
         return;

    case Alan_RST:                        // Process a Reset Packet
         AlanSta.AlanFlag.AReset = 1;     // Remote requested a Reset
         break;

    case Alan_REP:                        // Process a Report Request Packet
         AlanSta.AlanFlag.ARep = 1;       // Remote requested a Report
         return;                          // Packet Process Complete

    case Alan_CANA:                       // Process a CANA Packet
         AlanSta.AlanFlag.AAlrt = 0;      // Cancel Alert
         break;

    case Alan_ACK:                        // Process a ACK Packet
         AlanSta.AlanFlag.ARxAck = 1;     // Remote has Acknowledge the Packet
         return;                          // Packet Process Complete

    case Alan_A_ID:                       // Process a Identification Request Packet
         AlanDevId();                     // Send Identification String
         return;                          // Packet Process Complete

    default:
         AlanCmd(Alan_NEG);               // Send Back a NEG Packet
      return;                             // Do Nothing with Unknow type Packet
    }

    AlanCmd(Alan_ACK);                    // Send Back a ACK Packet
/*}}}  */

        return;
    }


    //---------------------------------
    // ALAN PACKET IS FOR PASS TROUGH
    //---------------------------------
    if((ARxSrcNET==AlanMyNET && ARxDestNET!=AlanMyNET) ||
       (ARxSrcNET!=AlanMyNET && ARxDestNET==AlanMyNET) ) {
       AlanBufSize--;                            // Remote checksum
       Acksum = (255-AlanBuffer[AlanBufSize]);
       AlanFooter();
    }
}
/*}}}  */

/*}}}  */


/*{{{  Main*/

void main() {
    unsigned char i;

    OPTION = 0xD3;              // No Pull-up, timer 1:16
	INTCON = 0b11000000;        // Interrupt enable
	PIE1   = 0b00100000;        // Serial rx int. enable
	PIE2   = 0b00000000;
	T1CON  = 0x39;              // 1:8 timer1 enabled (8uS timer)
    ADCON0 = 0x81;              // A/D in operation FOSC/32 (20Mhz compatible)
    ADCON1 = 0x80;              // All analog port enabled
    TRISA  = 0xFF;              // Input (Analog port + 1 digital)
	TRISB  = 0xE1;	            // I/O (with digital input 1-3)
	TRISC  = 0x9F;              // RS-485 port
    AlanInit();                 // Setup Serial Port for Alan
    PacketInit();
    
    TMR1L=0;                    // Wait .2sec
    TMR1H=0;
    while((TMR1H&128)==0);

	ATxDestID=0;                // Send identify packet
	ATxDestNET=AlanMyNET;
    AlanDevId();
    p=0;

    while (1) {
        if(TMR1H & 128) LED=1; else LED=0;
        AlanPoll();                 // Service Alan Network Command
        PacketIn_Update();          // Service packet receive
        /*{{{  Receive AX25 UI packet*/

// ADD RECEIVED BYTE TO BUFFER, CHECK FOR OVERFLOW
//--------------------------------------------------
    if(PacketIn_Flag&PKTIN_NEWBYTE) {
        PacketIn_Flag&=(~PKTIN_NEWBYTE);
        PktBuf[p++] = PacketIn_Byte;
        if(p==PKTBUFSIZE) p--;
    }


// RESET RECEIVER, FLUSH PACKET BUFFER
//--------------------------------------------
    if(PacketIn_Flag&PKTIN_RESET) {
        PacketIn_Flag&=(~PKTIN_RESET);
        PacketInit();
        p=0;
    }

// PACKET VALID, INJECT UI DATA TO ALAN
//-------------------------------------------------------
    if(PacketIn_Flag&PKTIN_DONE) {
        PacketIn_Flag&=(~PKTIN_DONE);
        p-=2;                             // Remove AX25 CRC
        for(tmp=0; tmp<6; tmp++)          // Dest is ALAN
            if(PktBuf[tmp]!=Pkt_Header[tmp]) p=0;
        if((PktBuf[6]&0x1F)!=0)     p=0;  // SSID is 0
        if((PktBuf[13]&0x01)!=1)    p=0;  // End of address (no digipeater path)
        if((PktBuf[14]&0xEF)!=0x03) p=0;  // UI control
        if(PktBuf[15]!=0xF0)        p=0;  // PID is 11110000 (no protocol)
        if(PktBuf[16]!=0xFF)        p=0;  // Alan start flag (sended in ax25)
        if((p-17)>ALANBUFFER_SIZE)    p=0;  // Verify to not overflow alan buffer

        if(p!=0) {
            Acksum = 0;                   // Verify checksum
            for(tmp=0; tmp<(p-17); tmp++) {
                AlanBuffer[tmp]=PktBuf[tmp+17];
                Acksum+=AlanBuffer[tmp];
            }
            AlanBufSize = p-17;                // Set size of alan packet

            AlanSta.AlanFlgReg = 0;	        // Clear Alan Flags
            AlanSta.AlanFlag.AlanRx = 1;    // One frame in alan buffer
            p=0;

        }
    }
/*}}}  */

        /*{{{  Alan data packet processing*/
         
//------------------------------------------------
// DATA PACKET PROCESSING
// !Ixx = set Alan ID
// !Nxx = set Alan NET
//------------------------------------------------
	if(AlanSta.AlanFlag.AValid)	{
	    AlanSta.AlanFlag.AValid  = 0;  // Remove Alan Valid Flag (Receive is ReEnable)

        if(AdtaSize=4 && AlanBuffer[6]=='!') {

            i = (16*Hex(AlanBuffer[8]))  + Hex(AlanBuffer[9]);
	        switch(AlanBuffer[7]) {
	            case 'I': EEPromWrite(EE_ALANID, i); AlanMyID=i; break;
	            case 'N': EEPromWrite(EE_ALANNET, i); AlanMyNET=i; break;
            }
        }
	}
/*}}}  */

        /*{{{  Alan Report Packet Processing*/
//------------------------------------------------
// Report CPU status:
//
//------------------------------------------------

    if(AlanSta.AlanFlag.ARep) {
        AlanSta.AlanFlag.ARep = 0;  // Report Packet is Processed

	    AlanHeader(Alan_STA);       // Send Status Packet
	    Transmit(10);               // Send Size of data field
        ADCRead(0);
        Transmit(ADRESH);           // ANALOG PORT #0
        Transmit(ADRESL);
        ADCRead(1);
        Transmit(ADRESH);           // ANALOG PORT #1
        Transmit(ADRESL);
        ADCRead(2);
        Transmit(ADRESH);           // ANALOG PORT #2
        Transmit(ADRESL);
        ADCRead(3);
        Transmit(ADRESH);           // ANALOG PORT #3
        Transmit(ADRESL);
        ADCRead(4);
        Transmit(ADRESH);           // ANALOG PORT #4
        Transmit(ADRESL);
	 	AlanFooter();               // Packet is now Complete
    }
/*}}}  */

    }
}
/*}}}  */

